#!/usr/bin/env python3
import subprocess, json
from sysrepo_api import sysrepo_api3 as api
from configobj import ConfigObj
from time import sleep

class SysrepoNTP(api.SysrepoAPI):

    def __init__(self,
                 app_name="ntp",
                 log_file=None,
                 log_level=None,
                 console_log=False,
                 namespace="/ucpe:config/ntp:ntp",
                 indexes={"server": "name"},
                 run_as_subscriber=False,
                 priority=0
                 ):
        api.SysrepoAPI.__init__(
            self,
            app_name=app_name,
            log_file=log_file,
            log_level=log_level,
            console_log=console_log,
            namespace=namespace,
            run_as_subscriber=run_as_subscriber,
            indexes=indexes,
            priority=priority#,
            #service_list=service_list
        )

    def apply_ntp2system(self, timezone="", ntp_servers = []):
        retries = 3
        attempt =0
        while attempt < retries:
            try:
                subprocess.check_output(['timedatectl', 'set-ntp', 'false'])
            except:
                self.logger.error("Cant stop ntp!")
                sleep(1)
                continue
            else:
                break
            finally:
                attempt += 1
        self.logger.debug('Update NTP servers')
        cfg_obj = ConfigObj("/etc/systemd/timesyncd.conf")
        time = {'NTP': " ".join(ntp_servers)}
        cfg_obj['Time'] = time
        cfg_obj.write()
        if timezone != "":
            self.logger.debug('Update Timezone')
            attempt =0
            while attempt < retries:
                try:
                    subprocess.check_output(['timedatectl', 'set-timezone', timezone])
                except:
                    self.logger.error("Cant set timezone!")
                    sleep(1)
                    continue
                else:
                    break
                finally:
                    attempt += 1
            
        attempt =0
        while attempt < retries:
            try:
                subprocess.check_output(['timedatectl', 'set-ntp', 'true'])
            except:
                self.logger.error("Cant start NTP!")
                sleep(1)
                continue
            else:
                break
            finally:
                attempt += 1
        

    def apply_startup(self):
        self.logger.debug( 'Apply STARTUP config for NTP' )
        sys_conf = self.get_startup_config(full_config=False, full_path=False)

        self.logger.debug( 'Startup config : %s' % json.dumps( sys_conf ) )
        timezone = ""
        ntp_servers = []

        if 'timezone' in sys_conf:
            timezone = sys_conf['timezone']

        if 'server' in sys_conf:
            for server in sys_conf['server']:
                ntp_servers.append(server['name'])

        self.apply_ntp2system(timezone, ntp_servers)

        return True

    def apply_running(self, change_set):
        self.logger.debug('Apply RUNNING config for NTP')
        json_data = self.get_running_config(full_path=False)
        timezone = ""
        ntp_servers = []

        if 'server' in json_data:
            for server in json_data['server']:
                ntp_servers.append(server['name'])

        for change in change_set:
            if not change["xpath"].startswith(self.namespace):
                continue
            if not 'operation' in change:
                self.logger.error( 'Malformed change set')
                continue

            self.logger.debug( 'Operation -->> %s' % change[ 'operation' ] )
            self.logger.debug( 'XPath -->> %s' % change[ 'xpath' ] )

            if change[ 'operation' ] == 'CREATED':
                self.logger.debug( 'Created - %s' % change[ 'xpath' ] )
                #Add new URI repo-server
                if change["xpath"].find("ntp/server") != -1 and change["xpath"].find("/name") != -1:
                    ntp_servers.append(change[ 'new_value' ])

            elif change[ 'operation'] == 'DELETED':
                #Remove one URI from repo-server(s)
                if change["xpath"].find("ntp/server") != -1 and change["xpath"].find("/name") != -1:
                    self.logger.debug( 'Deleted - %s, \'%s\'' % (change[ 'xpath' ], change[ 'old_value' ]) )
                    if change[ 'old_value' ] in ntp_servers:
                        ntp_servers.remove(change[ 'old_value' ])

            elif change[ 'operation' ] == 'MODIFIED':
                if "timezone" in change["xpath"]:
                    self.logger.debug('Modified  - %s: %s -> %s' % (change[ 'xpath' ], change[ 'old_value' ], change[ 'new_value' ]) )
                    timezone = change['new_value']

            elif change[ 'operation' ] == 'MOVED':
                self.logger.debug('Moved  - %s' % change['xpath'])

            else:
                self.logger.error( 'Unknown operation \'%s\'' %  change[ 'operation' ])

        self.apply_ntp2system(timezone, ntp_servers)

        return True

    def apply_config(self, change_set=None, apply_startup=False):
        ret = False
        if apply_startup:
            ret = self.apply_startup()
        else:
            ret = self.apply_running(change_set)

        return ret

    def set_timezone(self, timezone):
        self.set_item("{}/timezone".format(self.namespace), timezone)

    def get_timezone(self, ):
        return self.get_item("{}/timezone".format(self.namespace))

    def unset_timezone(self):
        self.delete_item("{}/timezone".format(self.namespace))

    def set_ntp_server(self, name):
        self.set_item("{}/server[name='{}']".format(self.namespace, name), name)

    def get_ntp_server(self, name):
        return self.get_item("{}/server[name='{}']".format(self.namespace, name))

    def unset_ntp_server(self, name):
        self.delete_item("{}/server[name='{}']".format(self.namespace, name))

    def get_ntp_servers(self):
        config = self.get_running_config(full_path=False)
        config_indexed = {}
        if 'server' not in config:
            return config_indexed
        for server in config['server']:
            config_indexed[server['name']] = server
        return config_indexed
